/*******************************************************************************
 *   Copyright 1998 Integrated Device Technology, Inc.
 *   All right reserved.
 *
 *   This program is proprietary and confidential information of
 *   IDT Corp. and may be used and disclosed only as authorized
 *   in a license agreement controlling such use and disclosure. 
 *
 *   IDT reserves the right to make any changes to
 *   the product herein to improve function or design.
 *   IDT does not assume any liability arising out of     
 *   the application or use of the product herein.
 *
 *   WARNING: The unlicensed shipping, mailing, or carring of this
 *   technical data outside the United States, or the unlicensed
 *   disclosure, by whatever means, through visits abroad, or the
 *   unlicensed disclosure to foreign national in the United States,
 *   may violate the United States criminal law.
 *
 *   File Name                 : idt71v632s6.v
 *   Function                  : 64Kx32 pipeline burst Static RAM
 *   Simulation Tool/Version   : Verilog-XL 2.5
 *      
 ******************************************************************************/
      
/*******************************************************************************
 * Module Name: idt71v632s6
 *
 * Notes                     : This model is believed to be functionally
 *                             accurate.  Please direct any inquiries to
 *                             IDT SRAM Applications at: sramhelp@idt.com
 *                               
 *******************************************************************************/
`timescale 1ns/100ps

module idt71v632s6(A, D, oe_, ce_, cs0, cs1_, lbo_, 
                   gw_, bwe_, bw4_, bw3_, bw2_, bw1_,
                   adsp_, adsc_, adv_, clk);
initial
begin
   $write("\n********************************************************\n");
   $write("   idt71v632s6 			        \n");
   $write("   Rev: 9_21_98                              \n"); 
   $write("   copyright 1996,1997,1998 by IDT, Inc.     \n");
   $write("********************************************************\n\n");
end

inout [31:0] D;
input [15:0] A;
input oe_, ce_, cs0, cs1_, lbo_, gw_, bwe_, bw4_, bw3_, bw2_, bw1_,
      adsp_, adsc_, adv_, clk;

//internal registers for data, address, burst counter
reg [31:0] dout, din;
reg [15:0] reg_addr;
reg [1:0] brst_cnt;

wire[15:0] m_ad;
wire[31:0] data_out;

parameter regdelay = 1;

reg wr_b1_, wr_b2_, wr_b3_, wr_b4_, deselr, deselrr; 

wire check_data =  (~adsc_ & adsp_ & ~ce_ & cs0 & ~cs1_ 
                           & (~gw_ | ~bwe_ & (~bw1_ | ~bw2_ | ~bw3_ | ~bw4_)))
                 | (~deselr & adsc_ & (adsp_ | ce_) 
                            & (~gw_ | ~bwe_ & (~bw1_ | ~bw2_ | ~bw3_ | ~bw4_)));

wire check_addr =   (~adsp_ & ~ce_ & cs0 & ~cs1_)
                  | ( adsp_ & ~adsc_ & ~ce_ & cs0 & ~cs1_);

specify
specparam

//Clock Parameters
   tCYC  = 12,  //clock cycle time
   tCH   = 4.5, //clock high time
   tCL   = 4.5, //clock low time
//Output Parameters
   tCD   = 6,   //clk to data
   tCDC  = 2,   //output hold from clock
   tCLZ  = 0,   //CLK to output Low-Z
   tCHZ  = 5,   //CLK to output Hi-Z
   tOE   = 5,   //OE to output valid
   tOHZ  = 5,   //OE to output Hi-Z
   tOLZ  = 0,   //OE to output Hi-Z
//Set up times   
   tSA   = 2.5, //address set-up
   tSS   = 2.5, //address status set-up
   tSD   = 2.5, //data set-up
   tSW   = 2.5, //write set-up
   tSAV  = 2.5,
   tSC   = 2.5,
//Hold times
   tHA   = 0.5, //Address hold
   tHS   = 0.5, //address status set-up
   tHD   = 0.5, //data hold
   tHW   = 0.5, //write set-up
   tHAV  = 0.5,
   tHC   = 0.5;

   (oe_ *> D) = (tOE,tOE,tOHZ,tOLZ,tOHZ,tOLZ); //(01,10,0z,z1,1z,z0)
   (clk *> D) = (tCD,tCD,tCHZ,tCLZ,tCHZ,tCLZ); //(01,10,0z,z1,1z,z0)

//timing checks

   $period(posedge clk, tCYC );
   $width (posedge clk, tCH );
   $width (negedge clk, tCL );

   $setup(adsp_, posedge clk, tSS);
   $setup(adsc_, posedge clk, tSS);
   $setup(adv_, posedge clk, tSAV);
   $setup(gw_, posedge clk, tSW);
   $setup(bwe_, posedge clk, tSW);
   $setup(bw1_, posedge clk, tSW);
   $setup(bw2_, posedge clk, tSW);
   $setup(bw3_, posedge clk, tSW);
   $setup(bw4_, posedge clk, tSW);
   $setup(ce_, posedge clk, tSC);
   $setup(cs0, posedge clk, tSC);
   $setup(cs1_, posedge clk, tSC);
   
   $setup(A, posedge clk &&& check_addr, tSA);
   $setup(D, posedge clk &&& check_data, tSD);

   $hold(adsp_, posedge clk, tHS);
   $hold(adsc_, posedge clk, tHS);
   $hold(adv_, posedge clk, tHAV);
   $hold(gw_, posedge clk, tHW);
   $hold(bwe_, posedge clk, tHW);
   $hold(bw1_, posedge clk, tHW);
   $hold(bw2_, posedge clk, tHW);
   $hold(bw3_, posedge clk, tHW);
   $hold(bw4_, posedge clk, tHW);
   $hold(ce_, posedge clk, tHC);
   $hold(cs0, posedge clk, tHC);
   $hold(cs1_, posedge clk, tHC);
 
   $hold(A, posedge clk &&& check_addr, tHA);
   $hold(D, posedge clk &&& check_data, tHD);

endspecify

//
//memory array
reg [7:0] memb1[0:65535], memb2[0:65535], memb3[0:65535], memb4[0:65535];

wire doe, baddr1, baddr0, dsel;

//Output buffers: using a bufif1 has the same effect as...
//
//	assign D = doe ? data_out : 32'hzzzzzzzz;
//	
//It was coded this way to support SPECIFY delays in the specparam section.

bufif1 (D[0],data_out[0],doe);
bufif1 (D[1],data_out[1],doe);
bufif1 (D[2],data_out[2],doe);
bufif1 (D[3],data_out[3],doe);
bufif1 (D[4],data_out[4],doe);
bufif1 (D[5],data_out[5],doe);
bufif1 (D[6],data_out[6],doe);
bufif1 (D[7],data_out[7],doe);

bufif1 (D[8],data_out[8],doe);
bufif1 (D[9],data_out[9],doe);
bufif1 (D[10],data_out[10],doe);
bufif1 (D[11],data_out[11],doe);
bufif1 (D[12],data_out[12],doe);
bufif1 (D[13],data_out[13],doe);
bufif1 (D[14],data_out[14],doe);
bufif1 (D[15],data_out[15],doe);

bufif1 (D[16],data_out[16],doe);
bufif1 (D[17],data_out[17],doe);
bufif1 (D[18],data_out[18],doe);
bufif1 (D[19],data_out[19],doe);
bufif1 (D[20],data_out[20],doe);
bufif1 (D[21],data_out[21],doe);
bufif1 (D[22],data_out[22],doe);
bufif1 (D[23],data_out[23],doe);

bufif1 (D[24],data_out[24],doe);
bufif1 (D[25],data_out[25],doe);
bufif1 (D[26],data_out[26],doe);
bufif1 (D[27],data_out[27],doe);
bufif1 (D[28],data_out[28],doe);
bufif1 (D[29],data_out[29],doe);
bufif1 (D[30],data_out[30],doe);
bufif1 (D[31],data_out[31],doe);

assign doe = ~deselr & ~deselrr & ~oe_ & wr_b1_ & wr_b2_ & wr_b3_ & wr_b4_; 

assign dsel = (ce_ | ~cs0 | cs1_);

always @(posedge clk)
begin
   if ( ~adsc_ || ( ~adsp_ && ~ce_ )) deselr <= dsel;
end

always @(posedge clk)
begin
   deselrr <= deselr;
end

//write enable generation
always @(posedge clk)
begin
   if (  (~adsc_ & adsp_ & ~ce_ & cs0 & ~cs1_ & (~gw_ | ~bwe_ & ~bw1_))
       | (~deselr & adsc_ & (adsp_ | ce_) & (~gw_ | ~bwe_ & ~bw1_)))
      wr_b1_ <= 0;
   else wr_b1_ <= 1;
   if (  (~adsc_ & adsp_ & ~ce_ & cs0 & ~cs1_ & (~gw_ | ~bwe_ & ~bw2_))
       | (~deselr & adsc_ & (adsp_ | ce_) & (~gw_ | ~bwe_ & ~bw2_)))
      wr_b2_ <= 0;
   else wr_b2_ <= 1;
   if (  (~adsc_ & adsp_ & ~ce_ & cs0 & ~cs1_ & (~gw_ | ~bwe_ & ~bw3_))
       | (~deselr & adsc_ & (adsp_ | ce_) & (~gw_ | ~bwe_ & ~bw3_)))
      wr_b3_ <= 0;
   else wr_b3_ <= 1;
   if (  (~adsc_ & adsp_ & ~ce_ & cs0 & ~cs1_ & (~gw_ | ~bwe_ & ~bw4_))
       | (~deselr & adsc_ & (adsp_ | ce_) & (~gw_ | ~bwe_ & ~bw4_)))
      wr_b4_ <= 0;
   else wr_b4_ <= 1;
end
       
//input address register
always @(posedge clk)
begin
   if (  (~adsp_ & ~ce_ & cs0 & ~cs1_)
       | ( adsp_ & ~adsc_ & ~ce_ & cs0 & ~cs1_)) reg_addr[15:0] <= A[15:0];
end

// burst counter
//------------------
always @(posedge clk)
begin
   if (lbo_ & (  (~adsp_ & ~ce_ & cs0 & ~cs1_)
               | ( adsp_ & ~adsc_ & ~ce_ & cs0 & ~cs1_))) brst_cnt <= 0;
   else if (~lbo_ & (  (~adsp_ & ~ce_ & cs0 & ~cs1_)
                     | ( adsp_ & ~adsc_ & ~ce_ & cs0 & ~cs1_))) brst_cnt <= A[1:0];
   else if ((adsp_ | ce_) & adsc_ & ~adv_) brst_cnt <= brst_cnt + 1;
end

//determine the memory address

assign baddr1 = lbo_ ? (brst_cnt[1] ^ reg_addr[1]) : brst_cnt[1];
assign baddr0 = lbo_ ? (brst_cnt[0] ^ reg_addr[0]) : brst_cnt[0];

assign #regdelay m_ad[15:0] = {reg_addr[15:2], baddr1, baddr0};

//data output register
//--------------------
always @(posedge clk)
begin
   dout[31:24] <= memb4[m_ad];
   dout[23:16] <= memb3[m_ad];
   dout[15:8]  <= memb2[m_ad];
   dout[7:0]   <= memb1[m_ad];
end

assign data_out = dout;

//data input register
//--------------------
always @(posedge clk)
begin
   din <= #regdelay D;
end

// write to ram
//-------------
wire #1 wrb1 = ~wr_b1_ & ~clk;
wire #1 wrb2 = ~wr_b2_ & ~clk;
wire #1 wrb3 = ~wr_b3_ & ~clk;
wire #1 wrb4 = ~wr_b4_ & ~clk;

always @(clk)
begin
   if (wrb1) memb1[m_ad] = din[7:0];
   if (wrb2) memb2[m_ad] = din[15:8];
   if (wrb3) memb3[m_ad] = din[23:16];
   if (wrb4) memb4[m_ad] = din[31:24];
end

endmodule
